@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -93,6 +93,10 @@ channel_movie = { }
 # Is there a video being displayed fullscreen?
 fullscreen = False
 
+# Movie channels that had a hide operation since the last interaction took
+# place.
+reset_channels = set()
+
 
 def early_interact():
     """
@@ -186,6 +190,14 @@ def render_movie(channel, width, height):
     return rv
 
 
+def default_play_callback(old, new):  # @UnusedVariable
+
+    renpy.audio.music.play(new._play, channel=new.channel, loop=True, synchro_start=True)
+
+    if new.mask:
+        renpy.audio.music.play(new.mask, channel=new.mask_channel, loop=True, synchro_start=True)
+
+
 class Movie(renpy.display.core.Displayable):
     """
     :doc: movie
@@ -222,7 +234,7 @@ class Movie(renpy.display.core.Displayable):
 
     `mask_channel`
         The channel the alpha mask video is played on. If not given,
-        defaults to `channel`_mask. (For example, if `channel` is "sprite",
+        defaults to `channel`\ _mask. (For example, if `channel` is "sprite",
         `mask_channel` defaults to "sprite_mask".)
 
     `image`
@@ -232,6 +244,33 @@ class Movie(renpy.display.core.Displayable):
         sprites.) Users can also choose to fall back to this image as a
         preference if video is too taxing for their system.
 
+    ``play_callback``
+        If not None, a function that's used to start the movies playing.
+        (This may do things like queue a transition between sprites, if
+        desired.) It's called with the following arguments:
+
+        `old`
+            The old Movie object, or None if the movie is not playing.
+        `new`
+            The new Movie object.
+
+        A movie object has the `play` parameter available as ``_play``,
+        while the ``channel``, ``mask``, and ``mask_channel`` fields
+        correspond to the given parameters.
+
+        Generally, this will want to use :func:`renpy.music.play` to start
+        the movie playing on the given channel, with synchro_start=True.
+        A minimal implementation is::
+
+            def play_callback(old, new):
+
+                renpy.music.play(new._play, channel=new.channel, loop=True, synchro_start=True)
+
+                if new.mask:
+                    renpy.music.play(new.mask, channel=new.mask_channel, loop=True, synchro_start=True)
+
+
+
     This displayable will be transparent when the movie is not playing.
     """
 
@@ -244,6 +283,8 @@ class Movie(renpy.display.core.Displayable):
 
     image = None
 
+    play_callback = None
+
     def ensure_channel(self, name):
 
         if name is None:
@@ -252,9 +293,14 @@ class Movie(renpy.display.core.Displayable):
         if renpy.audio.music.channel_defined(name):
             return
 
-        renpy.audio.music.register_channel(name, renpy.config.movie_mixer, loop=True, stop_on_mute=False, movie=True)
+        if self.mask:
+            framedrop = True
+        else:
+            framedrop = False
 
-    def __init__(self, fps=24, size=None, channel="movie", play=None, mask=None, mask_channel=None, image=None, **properties):
+        renpy.audio.music.register_channel(name, renpy.config.movie_mixer, loop=True, stop_on_mute=False, movie=True, framedrop=framedrop)
+
+    def __init__(self, fps=24, size=None, channel="movie", play=None, mask=None, mask_channel=None, image=None, play_callback=None, **properties):
         super(Movie, self).__init__(**properties)
 
         global auto_channel_serial
@@ -280,6 +326,8 @@ class Movie(renpy.display.core.Displayable):
 
         self.image = renpy.easy.displayable_or_none(image)
 
+        self.play_callback = play_callback
+
         if (self.channel == "movie") and (renpy.config.hw_video) and renpy.mobile:
             raise Exception("Movie(channel='movie') doesn't work on mobile when config.hw_video is true. (Use a different channel argument.)")
 
@@ -300,6 +348,9 @@ class Movie(renpy.display.core.Displayable):
         if self._play:
             channel_movie[self.channel] = self
 
+            if st == 0:
+                reset_channels.add(self.channel)
+
         playing = renpy.audio.music.get_playing(self.channel)
 
         if self.size is None:
@@ -340,12 +391,13 @@ class Movie(renpy.display.core.Displayable):
         else:
             old_play = old._play
 
-        if self._play != old_play:
+        if (self._play != old_play) or renpy.config.replay_movie_sprites:
             if self._play:
-                renpy.audio.music.play(self._play, channel=self.channel, loop=True, synchro_start=True)
 
-                if self.mask:
-                    renpy.audio.music.play(self.mask, channel=self.mask_channel, loop=True, synchro_start=True)
+                if self.play_callback is not None:
+                    self.play_callback(old, self)
+                else:
+                    default_play_callback(old, self)
 
             else:
                 renpy.audio.music.stop(channel=self.channel)
@@ -375,6 +427,8 @@ def playing():
         if renpy.audio.music.get_playing(channel):
             return True
 
+    return
+
 
 def update_playing():
     """
@@ -387,7 +441,9 @@ def update_playing():
 
         old = old_channel_movie.get(c, None)
 
-        if old is not m:
+        if (c in reset_channels) and renpy.config.replay_movie_sprites:
+            m.play(old)
+        elif old is not m:
             m.play(old)
 
     for c, m in old_channel_movie.items():
@@ -395,6 +451,7 @@ def update_playing():
             m.stop()
 
     renpy.game.context().movie = dict(channel_movie)
+    reset_channels.clear()
 
 
 def frequent():
