@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -31,6 +31,24 @@ Color = renpy.color.Color
 color = renpy.color.Color
 
 
+def lookup_displayable_prefix(d):
+    """
+    Given `d`, a string given a displayable, returns the displayale it
+    corresponds to or None if it it does notr correspond to one.
+    """
+
+    prefix, colon, arg = d.partition(":")
+
+    if not colon:
+        return None
+
+    fn = renpy.config.displayable_prefix.get(prefix, None)
+    if fn is None:
+        return None
+
+    return displayable(fn(arg))
+
+
 def displayable_or_none(d, scope=None, dynamic=True):
 
     if isinstance(d, renpy.display.core.Displayable):
@@ -44,6 +62,11 @@ def displayable_or_none(d, scope=None, dynamic=True):
             raise Exception("An empty string cannot be used as a displayable.")
         elif ("[" in d) and renpy.config.dynamic_images and dynamic:
             return renpy.display.image.DynamicImage(d, scope=scope)
+
+        rv = lookup_displayable_prefix(d)
+
+        if rv is not None:
+            return rv
         elif d[0] == '#':
             return renpy.store.Solid(d)
         elif "." in d:
@@ -85,6 +108,11 @@ def displayable(d, scope=None):
             raise Exception("An empty string cannot be used as a displayable.")
         elif ("[" in d) and renpy.config.dynamic_images:
             return renpy.display.image.DynamicImage(d, scope=scope)
+
+        rv = lookup_displayable_prefix(d)
+
+        if rv is not None:
+            return rv
         elif d[0] == '#':
             return renpy.store.Solid(d)
         elif "." in d:
@@ -108,7 +136,7 @@ def displayable(d, scope=None):
     raise Exception("Not a displayable: %r" % (d,))
 
 
-def dynamic_image(d, scope=None, prefix=None):
+def dynamic_image(d, scope=None, prefix=None, search=None):
     """
     Substitutes a scope into `d`, then returns a displayable.
 
@@ -119,6 +147,21 @@ def dynamic_image(d, scope=None, prefix=None):
     if not isinstance(d, list):
         d = [ d ]
 
+    def find(name):
+
+        if renpy.loader.loadable(name):
+            return True
+
+        if renpy.exports.image_exists(name):
+            return True
+
+        if lookup_displayable_prefix(name):
+            return True
+
+        if (len(d) == 1) and (renpy.config.missing_image_callback is not None):
+            if renpy.config.missing_image_callback(name):
+                return True
+
     for i in d:
 
         if not isinstance(i, basestring):
@@ -136,25 +179,30 @@ def dynamic_image(d, scope=None, prefix=None):
 
                 rv = renpy.substitutions.substitute(i, scope=scope, force=True, translate=False)[0]
 
-                if renpy.loader.loadable(rv):
+                if find(rv):
                     return displayable_or_none(rv)
 
-                if renpy.exports.image_exists(rv):
-                    return displayable_or_none(rv)
+                if search is not None:
+                    search.append(rv)
 
         else:
 
             rv = renpy.substitutions.substitute(i, scope=scope, force=True, translate=False)[0]
 
-            if renpy.loader.loadable(rv):
+            if find(rv):
                 return displayable_or_none(rv)
 
-            if renpy.exports.image_exists(rv):
-                return displayable_or_none(rv)
+            if search is not None:
+                search.append(rv)
 
     else:
 
-        return displayable_or_none(d[-1], dynamic=False)
+        rv = d[-1]
+
+        if find(rv):
+            return displayable_or_none(rv, dynamic=False)
+
+        return None
 
 
 def predict(d):
@@ -183,7 +231,7 @@ def split_properties(properties, *prefixes):
     If no prefix matches, an exception is thrown. (The empty string, "",
     can be used as the last prefix to create a catch-all dictionary.)
 
-    For example, this code splits properties beginning with text from
+    For example, this splits properties beginning with text from
     those that do not::
 
         text_properties, button_properties = renpy.split_properties("text_", "")
